// ** =======================================================
// Windows Sockets Database LookUp
// ----------------------------------------
// Get IP address for entered hostname and other way round.
// Possibility to choose between blocking and async mode.
//
// Note: Don't forget to include WSock32.lib.
// 
// Sunshine, February 2004
// Lighten-Sunrise@gmx.de
// www.sunshine2k.de
// ** =======================================================

#include <windows.h> 
#include <winsock.h>	// WinSock header file
#include "resource.h"

// say VC linker to set entrypoint to Winmain function
// needed cause I don't use c standard libraries
#pragma comment (linker,"/ENTRY:Winmain")


// ** Global Variables
HINSTANCE	hinst;		// store module handle
WSADATA		wsdata;		// store winsock info
HANDLE		hAsyncHost; // handle for asynchronous calls
PHOSTENT	pHostent;   // pointer to host entry structure
IN_ADDR		m_in;       // Internet address structure
char		bufhostent[MAXGETHOSTSTRUCT]; // hostent buffer for async call


// user-defined message for WSAAsyncGetHostByName
#define WM_ASYNC_HOSTBYNAME (WM_USER + 1)
// user-defined message for WSAAsyncGetHostByAddr
#define WM_ASYNC_HOSTBYADDR (WM_USER + 2)


// ** Main function for host lookup (blocking)
void LookUpHost(HWND hwnd)
{
	char buf[MAX_PATH];
	char result[MAX_PATH];
	ULONG IPAddy;
	
	GetDlgItemText(hwnd, ID_EDIT, buf, sizeof(buf));


	// user entered a dottet-decimal IP address?
	IPAddy = inet_addr(buf);
	if (IPAddy == INADDR_NONE) // no, user entered host name 
	
		pHostent = gethostbyname(buf);
	else
		pHostent = gethostbyaddr((char*)&IPAddy, 4, PF_INET);

	if (pHostent == NULL)
	{
		MessageBox(hwnd, "Host lookup failed!", "Error", MB_OK);
		return;
	}


	CopyMemory(&m_in, pHostent->h_addr,4);
	wsprintf(result, "%s ", pHostent->h_name );//inet_ntoa(pHostent->h_addr));
	SetDlgItemText(hwnd, ID_HOSTRES, result);
	wsprintf(result, "%s", inet_ntoa(m_in));
	SetDlgItemText(hwnd, ID_IPADRES, result);

	
}

void LookUpHostAsync(HWND hwnd)
{

	//char m_lpszHost[100];
	

	ULONG IPAddy;

	char buf[MAX_PATH];
	GetDlgItemText(hwnd, ID_EDIT, buf, sizeof(buf));

	IPAddy = inet_addr(buf);
	if (IPAddy == INADDR_NONE) // no, user entered host name 
	{
		hAsyncHost = WSAAsyncGetHostByName(hwnd, WM_ASYNC_HOSTBYNAME, 
			         buf, bufhostent, MAXGETHOSTSTRUCT);
	}
	else
	{
		hAsyncHost = WSAAsyncGetHostByAddr(hwnd, WM_ASYNC_HOSTBYADDR,
			(char *)&IPAddy, 4, PF_INET, bufhostent, MAXGETHOSTSTRUCT);
	}

	if (hAsyncHost == 0) MessageBox(hwnd, "Asynchronous host lookup failed!","Error", MB_OK);


}


BOOL DlgProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{

  switch(msg)
  {
  case WM_CLOSE:
	  {
		  EndDialog(hwnd, 0);
	  }
	  break;

  case WM_ASYNC_HOSTBYADDR:
  case WM_ASYNC_HOSTBYNAME: 
	  {
		  char result[MAX_PATH];

		  if (HIWORD(lparam) != 0) 
		  {
			  MessageBox(hwnd, "WM_ASYNC_HOSTBYNAME failed!", "Error", MB_OK);
			  return FALSE;
		  }

          
		  //MessageBox(hwnd, "WM_ASYNC_HOSTBYNAME received","",MB_OK);
		  hAsyncHost = 0;

		  // assign a hostent host entry pointer to the buffer
		  pHostent = (PHOSTENT)bufhostent;

		  CopyMemory(&m_in, pHostent->h_addr,4);
		  wsprintf(result, "%s ", pHostent->h_name );//inet_ntoa(pHostent->h_addr));
	      SetDlgItemText(hwnd, ID_HOSTRES, result);
	      wsprintf(result, "%s", inet_ntoa(m_in));
	      SetDlgItemText(hwnd, ID_IPADRES, result);


	  }
	  break;

  

	 

  case WM_COMMAND:
	  {
		  switch(wparam)
		  {
		  case ID_EXIT:
			  {
				  EndDialog(hwnd,0);
			  }
			  break;

		  case ID_HOST:
			  {
				  // checkbox for asynchronous lookup checked?
				  if (SendMessage(GetDlgItem(hwnd, ID_ASYNC), BM_GETSTATE, 0, 0) == BST_CHECKED)
				  {
					  // use asynchronous lookup
					  LookUpHostAsync(hwnd);
				  }
				  else
				  {
					  // use blocking mode
					  LookUpHost(hwnd);
				  }
			  }
			  break;

		  case ID_ABOUT:
			  {
				  MessageBox(hwnd, "-- Windows Sockets Host Lookup --   \n\n"
					               "'my first steps in the world of \n"
								   "windows sockets programming' \n\n"
								   "Sunshine, February 2k4 \n"
								   "www.sunshine2k.de",
								   "About...", MB_ICONINFORMATION);
			  }
			  break;

			  
		  default:break;

		  }
		  
	  }
	  break;

  default:break;
  }

  return FALSE;
}


int WINAPI Winmain()
{
	hinst = GetModuleHandle(0);

	// initialize winsock
	if (WSAStartup(MAKEWORD(1,1), &wsdata) != 0)
	{	  
		MessageBox(0,"Initializing Winsock failed!", "Bye",MB_OK);	  
		return 0;
	}

	DialogBox(hinst, MAKEINTRESOURCE(100), NULL, (DLGPROC)DlgProc);

	// clean up winsock
	WSACleanup();

	return 0;
}